unit DrawCtrl;

interface

uses 
  Windows, Messages;

function SetColorBrightness(SCBColor : COLORREF; SCBVal : Integer) : COLORREF;
procedure DrawColorButton(dblParam : LPARAM; BtnIco, IcoSize : Integer; BtnColor : COLORREF; BtnTextColor : COLORREF; BtnGradientColor : COLORREF = 0; Round : Byte = 0);
procedure DrawGradient(shdc : HDC; srect : TRect; clStart, clEnd : COLORREF);
procedure DrawColorCombo(CBXlparam : lParam; CBXIcon : DWORD);

implementation

function SetColorBrightness(SCBColor : COLORREF; SCBVal : Integer) : COLORREF;
var
  SC_B, SC_G, SC_R, SC_A : Integer;
  function CalcCVal(CVal : Integer) : Integer;
    begin
      result := (CVal * SCBVal) div 100;
      if (result > $FF) then
        result := $FF
      else
      if (result < $00) then
        result := $00;
    end;
begin
  SC_R := (SCBColor) and $FF;
  SC_G := (SCBColor shr  8) and $FF;
  SC_B := (SCBColor shr 16) and $FF;
  SC_A := (SCBColor shr 24) and $FF;
  SC_B := CalcCVal(SC_B);
  SC_G := CalcCVal(SC_G);
  SC_R := CalcCVal(SC_R);
  result := (SC_A shl 24) + (SC_B shl 16) + (SC_G shl 8) + (SC_R);
end;

procedure DrawColorButton(dblParam : LPARAM; BtnIco, IcoSize : Integer; BtnColor : COLORREF; BtnTextColor : COLORREF; BtnGradientColor : COLORREF = 0; Round : Byte = 0);
var
  hButtonDC    : HDC;
  ButtonRect,
  ButtonOldRect : TRect;
  hButtonWnd : HWND;

  BtnBrushTemplate       : TLogBrush;
  BtnRoundBrushTemplate  : TLogBrush;
  BtnHiBrushTemplate     : TLogBrush;
  BtnLoBrushTemplate     : TLogBrush;
  BtnVeryLoBrushTemplate : TLogBrush;

  BtnBrush       : HBRUSH;
  BtnRoundBrush  : HBRUSH;
  BtnHiBrush     : HBRUSH;
  BtnLoBrush     : HBRUSH;
  BtnVeryLoBrush : HBRUSH;

  BtnRoundPen  : HPEN;

  hButtonIconN, hButtonIconG, BtnRoundCount : Integer;

  BtnPC: array[0..$400] of char;

  BtnBkColor : COLORREF;
Label
  NoEdge;
begin

  hButtonIconN := LoadImage(hInstance, MAKEINTRESOURCE(BtnIco), IMAGE_ICON, IcoSize, IcoSize, LR_DEFAULTSIZE or LR_LOADTRANSPARENT or LR_LOADMAP3DCOLORS);
  hButtonIconG := LoadImage(hInstance, MAKEINTRESOURCE(BtnIco), IMAGE_ICON, IcoSize, IcoSize, LR_DEFAULTSIZE or LR_MONOCHROME);

  with BtnBrushTemplate do
    begin
      lbStyle := BS_SOLID;
      lbColor := BtnColor;
      lbHatch := 0;
    end;

  BtnBrush := CreateBrushIndirect(BtnBrushTemplate);

  with BtnHiBrushTemplate do
    begin
      lbStyle := BS_SOLID;
      lbColor := SetColorBrightness(BtnBrushTemplate.lbColor, 120);
      lbHatch := 0;
    end;

  BtnHiBrush := CreateBrushIndirect(BtnHiBrushTemplate);

  with BtnLoBrushTemplate do
    begin
      lbStyle := BS_SOLID;
      lbColor := SetColorBrightness(BtnBrushTemplate.lbColor, 60);
      lbHatch := 0;
    end;

  BtnLoBrush := CreateBrushIndirect(BtnLoBrushTemplate);

  with BtnVeryLoBrushTemplate do
    begin
      lbStyle := BS_SOLID;
      lbColor := SetColorBrightness(BtnBrushTemplate.lbColor, 30);
      lbHatch := 0;
    end;

  BtnVeryLoBrush := CreateBrushIndirect(BtnVeryLoBrushTemplate);

  hButtonWnd := PDRAWITEMSTRUCT(dblParam).hwndItem;

  hButtonDC := PDRAWITEMSTRUCT(dblParam).hDC;

  ButtonRect := PDRAWITEMSTRUCT(dblParam).rcItem;

  BtnBkColor := GetBkColor(hButtonDC);

  if (GetProp(hButtonWnd, 'MouseOver') = 1) then
    BtnGradientColor := SetColorBrightness(BtnGradientColor, 120);

  SetBkColor(hButtonDC, BtnBrushTemplate.lbColor);
  SetTextColor(hButtonDC, BtnTextColor);

  if (BtnGradientColor > 0) then
    begin
      if ((PDRAWITEMSTRUCT(dblParam).itemState and ODS_SELECTED) <> 0) then
        BtnGradientColor := SetColorBrightness(BtnGradientColor, 85);
      DrawGradient(hButtonDC, ButtonRect, BtnColor, BtnGradientColor);
    end
  else
    FillRect(hButtonDC, ButtonRect, BtnBrush);

  if ((PDRAWITEMSTRUCT(dblParam).itemState and ODS_FOCUS) <> 0) then
    begin
      ButtonRect.Left   := ButtonRect.Left   + 1;
      ButtonRect.Right  := ButtonRect.Right  - 1;
      ButtonRect.Top    := ButtonRect.Top    + 1;
      ButtonRect.Bottom := ButtonRect.Bottom - 1;
    end;

  ButtonOldRect := ButtonRect;

  if (Round > 0) then goto NoEdge;

  ButtonRect := ButtonOldRect;
  ButtonRect.Left   := ButtonRect.Left;
  ButtonRect.Right  := ButtonRect.Right - 1;
  ButtonRect.Top    := ButtonRect.Top;
  ButtonRect.Bottom := ButtonRect.Top   + 1;
  if ((PDRAWITEMSTRUCT(dblParam).itemState and ODS_SELECTED) <> 0) then
    FillRect(hButtonDC, ButtonRect, BtnLoBrush)
  else
    FillRect(hButtonDC, ButtonRect, BtnHiBrush);

  ButtonRect := ButtonOldRect;
  ButtonRect.Left   := ButtonRect.Left;
  ButtonRect.Right  := ButtonRect.Left   + 1;
  ButtonRect.Top    := ButtonRect.Top    + 1;
  ButtonRect.Bottom := ButtonRect.Bottom - 1;
  if ((PDRAWITEMSTRUCT(dblParam).itemState and ODS_SELECTED) <> 0) then
    FillRect(hButtonDC, ButtonRect, BtnLoBrush)
  else
    FillRect(hButtonDC, ButtonRect, BtnHiBrush);

  ButtonRect := ButtonOldRect;
  ButtonRect.Right  := ButtonRect.Right  - 1;
  ButtonRect.Left   := ButtonRect.Right  - 1;
  ButtonRect.Bottom := ButtonRect.Bottom - 1;
  ButtonRect.Top    := ButtonRect.Top    + 1;
  if not ((PDRAWITEMSTRUCT(dblParam).itemState and ODS_SELECTED) <> 0) then
    FillRect(hButtonDC, ButtonRect, BtnLoBrush);

  ButtonRect := ButtonOldRect;
  ButtonRect.Right  := ButtonRect.Right  - 1;
  ButtonRect.Left   := ButtonRect.Left   + 1;
  ButtonRect.Bottom := ButtonRect.Bottom - 1;
  ButtonRect.Top    := ButtonRect.Bottom - 1;
  if not ((PDRAWITEMSTRUCT(dblParam).itemState and ODS_SELECTED) <> 0) then
    FillRect(hButtonDC, ButtonRect, BtnLoBrush);

  ButtonRect := ButtonOldRect;
  ButtonRect.Right  := ButtonRect.Right;
  ButtonRect.Left   := ButtonRect.Right  - 1;
  ButtonRect.Bottom := ButtonRect.Bottom - 1;
  ButtonRect.Top    := ButtonRect.Top;
  if ((PDRAWITEMSTRUCT(dblParam).itemState and ODS_SELECTED) <> 0) then
    FillRect(hButtonDC, ButtonRect, BtnLoBrush)
  else
    FillRect(hButtonDC, ButtonRect, BtnVeryLoBrush);

  ButtonRect := ButtonOldRect;
  ButtonRect.Right  := ButtonRect.Right;
  ButtonRect.Left   := ButtonRect.Left;
  ButtonRect.Bottom := ButtonRect.Bottom;
  ButtonRect.Top    := ButtonRect.Bottom - 1;
  if ((PDRAWITEMSTRUCT(dblParam).itemState and ODS_SELECTED) <> 0) then
    FillRect(hButtonDC, ButtonRect, BtnLoBrush)
  else
    FillRect(hButtonDC, ButtonRect, BtnVeryLoBrush);

  NoEdge : ButtonRect := ButtonOldRect;

  if ((PDRAWITEMSTRUCT(dblParam).itemState and ODS_SELECTED) <> 0) then
    begin
      ButtonRect.Left := ButtonRect.Left + 2;
      ButtonRect.Top := ButtonRect.Top;
    end;

  SendMessage(hButtonWnd, WM_GETTEXT, $400, LongInt(@BtnPC[0]));

  if (BtnGradientColor > 0) then
    SetBkMode(hButtonDC, TRANSPARENT);

  if ((PDRAWITEMSTRUCT(dblParam).itemState and ODS_DISABLED) <> 0) then
    begin
      SetTextColor(hButtonDC, SetColorBrightness(BtnBrushTemplate.lbColor, 60));
      ButtonRect.Left := ButtonRect.Left + IcoSize + 5;
      ButtonRect.Top := ButtonRect.Top;
      DrawText(hButtonDC, @BtnPC[0], -1, ButtonRect, DT_SINGLELINE or DT_VCENTER or DT_CENTER);
      //       ( )
      if BtnPC = '' then
        begin
          ButtonRect.Left := ButtonRect.Left + 6;
          ButtonRect.Top := ButtonRect.Top + 6;
        end;
      DrawIconEx(hButtonDC, ButtonRect.Left - IcoSize, ButtonRect.Top + 5, hButtonIconG, IcoSize, IcoSize, 0, 0, DI_NORMAL);
      ButtonRect.Left := ButtonRect.Left;
      ButtonRect.Top := ButtonRect.Top;
      SetBkMode(hButtonDC, TRANSPARENT);
    end
  else
  if (BtnGradientColor > 0) then
    begin
      SetTextColor(hButtonDC, BtnTextColor);
      ButtonRect.Left := ButtonRect.Left + IcoSize + 5;
      ButtonRect.Top := ButtonRect.Top;
      if IcoSize = 0 then
        begin
          ButtonRect.Left := ButtonRect.Left - 4;
          ButtonRect.Top := ButtonRect.Top - 1;
        end;
      DrawText(hButtonDC, @BtnPC[0], -1, ButtonRect, DT_SINGLELINE or DT_VCENTER or DT_CENTER);
      DrawIconEx(hButtonDC, ButtonRect.Left - IcoSize, ButtonRect.Top + 5, hButtonIconN, IcoSize, IcoSize, 0, 0, DI_NORMAL);
      ButtonRect.Left := ButtonRect.Left;
      ButtonRect.Top := ButtonRect.Top;
      SetBkMode(hButtonDC, TRANSPARENT);
      SetTextColor(hButtonDC, BtnTextColor);
    end;

  DrawText(hButtonDC, @BtnPC[0], -1, ButtonRect, DT_SINGLELINE or DT_VCENTER or DT_CENTER);

  if (Round > 0) then
    begin
      with BtnRoundBrushTemplate do
        begin
          lbStyle := BS_HOLLOW;
          lbColor := 0;
          lbHatch := 0;
        end;
      BtnRoundBrush := CreateBrushIndirect(BtnRoundBrushTemplate);
      SelectObject(hButtonDC, BtnRoundBrush);

      ButtonRect := PDRAWITEMSTRUCT(dblParam).rcItem;

      BtnRoundPen := CreatePen(PS_SOLID, 1, SetColorBrightness(BtnBkColor, 110));
      SelectObject(hButtonDC, BtnRoundPen);
      RoundRect(hButtonDC, ButtonRect.Left, ButtonRect.Top, ButtonRect.Right, ButtonRect.Bottom, (Round * 2) + 2, Round + 2);
      DeleteObject(BtnRoundPen);

      ButtonRect.Left   := ButtonRect.Left   + 1;
      ButtonRect.Right  := ButtonRect.Right  - 1;
      ButtonRect.Top    := ButtonRect.Top    + 1;
      ButtonRect.Bottom := ButtonRect.Bottom - 1;

      if ((PDRAWITEMSTRUCT(dblParam).itemState and ODS_SELECTED) <> 0) then
        BtnRoundPen := CreatePen(PS_SOLID, 1, BtnVeryLoBrushTemplate.lbColor)
      else
        BtnRoundPen := CreatePen(PS_SOLID, 1, BtnLoBrushTemplate.lbColor);
      SelectObject(hButtonDC, BtnRoundPen);
      RoundRect(hButtonDC, ButtonRect.Left, ButtonRect.Top, ButtonRect.Right, ButtonRect.Bottom, (Round * 2), Round);
      DeleteObject(BtnRoundPen);

      ButtonRect := PDRAWITEMSTRUCT(dblParam).rcItem;

      if ((PDRAWITEMSTRUCT(dblParam).itemState and ODS_SELECTED) <> 0) then
        BtnRoundPen := CreatePen(PS_SOLID, 1, BtnLoBrushTemplate.lbColor)
      else
        BtnRoundPen := CreatePen(PS_SOLID, 1, BtnHiBrushTemplate.lbColor);

      SelectObject(hButtonDC, BtnRoundPen);
      for BtnRoundCount := 2 to (Round - 2) do
        begin
          ButtonRect.Left   := ButtonRect.Left   - 1;
          ButtonRect.Right  := ButtonRect.Right  + 1;
          ButtonRect.Top    := ButtonRect.Top    - 1;
          ButtonRect.Bottom := ButtonRect.Bottom + 1;

          RoundRect(hButtonDC, ButtonRect.Left, ButtonRect.Top, ButtonRect.Right, ButtonRect.Bottom, (Round * 2) + (BtnRoundCount * 2), Round + (BtnRoundCount * 2));
          RoundRect(hButtonDC, ButtonRect.Left, ButtonRect.Top, ButtonRect.Right, ButtonRect.Bottom, (Round * 2) + (BtnRoundCount * 2) - 1, Round + (BtnRoundCount * 2) - 1);
        end;
      DeleteObject(BtnRoundPen);

      SelectObject(hButtonDC, BtnBrush);
      DeleteObject(BtnRoundBrush);

      if ((PDRAWITEMSTRUCT(dblParam).itemState and ODS_FOCUS) <> 0) then
        begin
          ButtonRect := PDRAWITEMSTRUCT(dblParam).rcItem;
        end;
    end
  else
  if ((PDRAWITEMSTRUCT(dblParam).itemState and ODS_FOCUS) <> 0) then
    begin
      ButtonRect := PDRAWITEMSTRUCT(dblParam).rcItem;
      FrameRect(hButtonDC, ButtonRect, GetSysColorBrush(COLOR_3DDKSHADOW));

      ButtonRect.Left   := ButtonRect.Left   + 4;
      ButtonRect.Right  := ButtonRect.Right  - 4;
      ButtonRect.Top    := ButtonRect.Top    + 4;
      ButtonRect.Bottom := ButtonRect.Bottom - 4;
      DrawFocusRect(hButtonDC, ButtonRect);
    end;

  SetBkColor(hButtonDC, BtnBkColor);

  DeleteObject(BtnBrush);
  DeleteObject(BtnHiBrush);
  DeleteObject(BtnLoBrush);
  DeleteObject(BtnVeryLoBrush);
  DeleteObject(hButtonIconN);
  DeleteObject(hButtonIconG);
  ReleaseDC(hButtonWnd, hButtonDC);
end;

procedure DrawGradient(shdc : HDC; srect : TRect; clStart, clEnd : COLORREF);
type
  ExtByte = Real;
const
  sPenPoint : TPoint = (X : 1; Y : 1);
var
  TempRect : TRect;
  sBrushTemplate : TLogBrush;
  sPenTemplate : TLogPen;
  sBrush : HBRUSH;
  sPen : HPEN;
  dRed, dGreen, dBlue : Integer;
  CBlue, CGreen, CRed, CLine : ExtByte;
  CLinesX, CLinesY : Word;
  clStartR, clStartG, clStartB, clEndR, clEndG, clEndB : Byte;
begin
  clStartR := (clStart) and $FF;
  clStartG := (clStart shr 8) and $FF;
  clStartB := (clStart shr 16) and $FF;
  clEndR := (clEnd) and $FF;
  clEndG := (clEnd shr  8) and $FF;
  clEndB := (clEnd shr 16) and $FF;
  CLinesX := srect.Right  - srect.Left + 1;
  CLinesY := srect.Bottom - srect.Top + 1;
  with sBrushTemplate do
    begin
      lbStyle := BS_SOLID;
      lbColor := GetBkColor(shdc);
      lbHatch := 0;
    end;
  sBrush := CreateBrushIndirect(sBrushTemplate);
  TempRect := sRect;
  FillRect(shdc, TempRect, sBrush);
  dBlue  := (clEndB - clStartB);
  dGreen := (clEndG - clStartG);
  dRed := (clEndR - clStartR);
  CLine := srect.Top;
  while (CLine  < (CLinesY + srect.Top - 1)) do
    begin
      CBlue := (dBlue * (CLine - srect.Top + 1)) / CLinesY;
      CGreen := (dGreen * (CLine - srect.Top + 1)) / CLinesY;
      CRed := (dRed * (CLine - srect.Top + 1)) / CLinesY;
      CBlue := (clStartB + CBlue);
      CGreen := (clStartG + CGreen);
      CRed  := (clStartR + CRed);
      with sPenTemplate do
        begin
          lopnStyle := PS_SOLID;
          lopnWidth := sPenPoint;
          lopnColor := ((Round(CBlue) and $FF) shl 16) + ((Round(CGreen) and $FF) shl 8) + ((Round(CRed) and $FF));
        end;
      sPen := CreatePenIndirect(sPenTemplate);
      SelectObject(shdc, sPen);
      MoveToEx(shdc, srect.Left, Round(CLine), nil);
      LineTo(shdc, srect.Left + CLinesX - 1, Round(CLine));
      CLine := CLine + 1;
      DeleteObject(sPen);
    end;
  DeleteObject(sBrush);
end;

procedure DrawColorCombo(CBXlparam : lParam; CBXIcon : DWORD);
var
  hCbxWnd : HWND;
  hCbxIcon : Integer;
  hCbxDC : HDC;
  CbxRect : TRect;
  CbxPC : array [0..$400] of Char;
begin
  hCbxWnd := PDRAWITEMSTRUCT(CBXlParam).hwndItem;

  CbxRect := PDRAWITEMSTRUCT(CBXlParam).rcItem;

  hCbxDC := PDRAWITEMSTRUCT(CBXlParam).hDC;

  if (Integer(PDRAWITEMSTRUCT(CBXlParam).ItemID) > - 1) then
    begin
      hCbxIcon := LoadImage(hInstance, MAKEINTRESOURCE(CBXIcon), IMAGE_ICON, 16, 16, 0);

      if ((PDRAWITEMSTRUCT(CBXlParam).itemState and ODS_SELECTED) <> 0) then
        begin
          FillRect(hCbxDC, CbxRect, GetSysColorBrush(COLOR_HIGHLIGHT));
          DrawGradient(hCbxDC, CbxRect, SetColorBrightness(GetSysColor(COLOR_BTNFACE), 95), GetSysColor($FF0000FF and COLOR_BTNHIGHLIGHT));
          SetBkMode(hCbxDC, TRANSPARENT);
          SetTextColor(hCbxDC, GetSysColor(COLOR_MENUTEXT));
          PatBlt(hCbxDC, CbxRect.left, CbxRect.top, 1, CbxRect.bottom - CbxRect.top, DSTINVERT);
          PatBlt(hCbxDC, CbxRect.right - 1, CbxRect.top, 1, CbxRect.bottom - CbxRect.top, DSTINVERT);
          PatBlt(hCbxDC, CbxRect.left, CbxRect.top, CbxRect.right - CbxRect.left, 1, DSTINVERT);
          PatBlt(hCbxDC, CbxRect.left, CbxRect.bottom - 1, CbxRect.right - CbxRect.left, 1, DSTINVERT);
        end
      else
        begin
          FillRect(hCbxDC, CbxRect, GetSysColorBrush(COLOR_WINDOW));
          SetBkColor(hCbxDC, GetSysColor(COLOR_WINDOW));
          SetTextColor(hCbxDC, GetSysColor(COLOR_WINDOWTEXT));
        end;

      DrawIconEx(hCbxDC, CbxRect.Left + 2, CbxRect.Top + 1, hCbxIcon, 16, 16, 0, 0, DI_NORMAL);

      CbxRect.Left := CbxRect.Left + 22;

      SendMessage(hCbxWnd, CB_GETLBTEXT, PDRAWITEMSTRUCT(CBXlParam).ItemID, LongInt(@CbxPC[0]));

      DrawText(hCbxDC, @CbxPC[0], - 1, CbxRect, DT_SINGLELINE or DT_VCENTER);

      DeleteObject(hCbxIcon);
    end;

  ReleaseDC(hCbxWnd, hCbxDC);
end;

end.
